#define F_CPU 16000000L
#include <avr/io.h>
#include <util/delay.h>
#include "millis2560.h"
#include "UART0.h"

#define set_bit(value, bit) ( _SFR_BYTE(value) |= _BV(bit) )
#define clear_bit(value, bit) ( _SFR_BYTE(value) &= ~_BV(bit) )

int main(void) {
  UART0_init();
  millis2560_init();

  clear_bit(DDRJ, 0);     // 버튼 연결 핀을 입력으로 설정
  set_bit(DDRE, 4);       // LED 연결 핀 PE4를 출력으로 설정
  set_bit(DDRE, 5);       // LED 연결 핀 PE5를 출력으로 설정
  clear_bit(PORTE, 4);      // PB4 핀에 연결된 LED를 끈 상태로 시작
  clear_bit(PORTE, 5);      // PB5 핀에 연결된 LED를 끈 상태로 시작

  uint8_t state1 = 0, state2;     // LED 상태
  // 실행 시간
  unsigned long time_p = millis2560(), time_c;
  long loop_count = 0;      // while(1) 루프 실행 횟수

  while (1) {
    time_c = millis2560();      // 현재까지의 실행 시간
    loop_count++;

    if (time_c - time_p >= 1000) {  // 1초 이상이 경과한 경우
      time_p = time_c;      // 시작 시간 변경
      state1 = !state1;       // LED1 상태 반전
      if (state1) set_bit(PORTE, 4);  // LED1 켜기
      else clear_bit(PORTE, 4);     // LED1 끄기

      UART0_print(loop_count);    // while(1) 루프 실행 횟수 출력
      UART0_write('\n');
      loop_count = 0;
    }

    state2 = PINJ & 0x01;     // 버튼 상태 읽기
    if (state2) set_bit(PORTE, 5);    // LED2 켜기
    else clear_bit(PORTE, 5);     // LED2 끄기
  }

  return 0;
}
