#define F_CPU 16000000L
#include <avr/io.h>
#include <util/delay.h>
#include "millis2560.h"
#include "UART0.h"
#include "ADC.h"

#define set_bit(value, bit) ( _SFR_BYTE(value) |= _BV(bit) )
#define clear_bit(value, bit) ( _SFR_BYTE(value) &= ~_BV(bit) )

int main(void) {
  UART0_init();
  millis2560_init();
  ADC_init(0);        // AD 변환기 초기화

  set_bit(DDRB, 7);       // LED 연결 핀을 출력으로 설정
  clear_bit(PORTB, 7);      // LED를 끈 상태로 시작

  uint8_t state = 0;      // LED 상태
  unsigned long time_p = millis2560(), time_c;
  int interval = 1000;      // LED 점멸 간격

  while (1) {
    time_c = millis2560();

    if (time_c - time_p >= interval) {
      time_p = time_c;
      state = !state;       // LED1 상태 반전
      if (state) set_bit(PORTB, 7);   // LED1 켜기
      else clear_bit(PORTB, 7);   // LED1 끄기

      UART0_print("Current interval : ");
      UART0_print(interval);
      UART0_print("ms\n");
    }

    int read = read_ADC();    // 가변저항 읽기
    interval = (read >> 1) + 500;   // 점멸 간격으로 변환
  }

  return 0;
}
