#define F_CPU 16000000L
#include <avr/io.h>
#include <util/delay.h>
#include "UART0.h"
#include "ADC.h"

#define set_bit(value, bit) ( _SFR_BYTE(value) |= _BV(bit) )
#define clear_bit(value, bit) ( _SFR_BYTE(value) &= ~_BV(bit) )

#define THRESHOLD 28       // 경고 발생 온도

volatile int count = 0;       // 오버플로가 발생한 횟수
volatile uint8_t state = 0;     // LED 점멸 상태

ISR(TIMER0_OVF_vect) {
  count++;
  if (count == 32) {      // 오버플로 32회 발생 = 0.5초 경과
    count = 0;        // 인터럽트 발생 횟수 초기화
    state = !state;       // LED 상태 반전
    if (state) set_bit(PORTB, 7);     // LED 켜기
    else clear_bit(PORTB, 7);     // LED 끄기
  }
}

int main(void) {
  UART0_init();
  ADC_init(1);        // ADC 채널 1번 초기화

  TCCR0B |= (1 << CS02) | (1 << CS00);  // 분주비를 1024로 설정
  sei();              // 전역적으로 인터럽트 허용

  set_bit(DDRB, 7);
  clear_bit(PORTB, 7);

  int read, temperature;

  while (1) {
    read = read_ADC();
    temperature = (int)(read * 500.0 / 1023.0); // 온도 변환

    if (temperature < THRESHOLD) {
      TIMSK0 &= ~(1 << TOIE0);    // 오버플로 인터럽트 금지
      clear_bit(PORTB, 7);    // LED 끄기
    }
    else {
      TIMSK0 |= (1 << TOIE0);     // 오버플로 인터럽트 허용
    }

    UART0_print(temperature);
    UART0_write('\n');

    _delay_ms(1000);
  }

  return 0;
}
